package de.fhhannover.inform.trust.ifmapj.channel;

import java.io.InputStream;

import javax.net.ssl.KeyManager;
import javax.net.ssl.TrustManager;

import de.fhhannover.inform.trust.ifmapj.exception.CommunicationException;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapException;
import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;
import de.fhhannover.inform.trust.ifmapj.messages.EndSessionRequest;
import de.fhhannover.inform.trust.ifmapj.messages.NewSessionRequest;
import de.fhhannover.inform.trust.ifmapj.messages.NewSessionResult;
import de.fhhannover.inform.trust.ifmapj.messages.PublishRequest;
import de.fhhannover.inform.trust.ifmapj.messages.PurgePublisherRequest;
import de.fhhannover.inform.trust.ifmapj.messages.RenewSessionRequest;
import de.fhhannover.inform.trust.ifmapj.messages.SearchRequest;
import de.fhhannover.inform.trust.ifmapj.messages.SearchResult;
import de.fhhannover.inform.trust.ifmapj.messages.SubscribeRequest;

public class SsrcImpl extends Channel implements SSRC {
	
	/**
	 * the session-id received after the last call to {@link #newSession()}
	 */
	private String mSessionId;
	
	/**
	 * the publisher-id received after the last call to {@link #newSession()}
	 */
	private String mPublisherId;

	/**
	 * the max-poll-result-size received after the last call to {@link #newSession()},
	 * if any
	 */
	private Integer mMaxPollResultSize;
	
	/**
	 * KeyManager instances to initialize the SSL Context.
	 */
	private KeyManager[] mKeyManagers;
	
	/**
	 * TrustManager instances to initialize the SSL Context.
	 */
	private TrustManager[] mTrustManagers;

	/**
	 * Basic-Auth constructor.
	 * 
	 * @param url
	 * @param user
	 * @param pass
	 * @param tms
	 * @throws InitializationException
	 */
	public SsrcImpl(String url, String user, String pass, TrustManager[] tms)
			throws InitializationException {
		super(url, user, pass, tms);
		mTrustManagers = tms;
	}

	/**
	 * Certificate-based constructor.
	 * 
	 * @param url
	 * @param kms
	 * @param tms
	 * @throws InitializationException
	 */
	public SsrcImpl(String url, KeyManager[] kms, TrustManager[] tms)
			throws InitializationException {
		super(url, kms, tms);
		mKeyManagers = kms;
		mTrustManagers = tms;
	}
	
	@Override
	public void newSession(Integer maxPollResSize)
			throws IfmapErrorResult, IfmapException {
		
		NewSessionRequest nsreq = mRequestFactory.createNewSessionReq();
		
		nsreq.setMaxPollResultSize(maxPollResSize);
		
		InputStream isreq = mMarshaller.marshalNewSessionRequest(nsreq);
		InputStream isres = doHttpRequest(isreq);
		
		
		NewSessionResult nsres = mUnmarshaller.unmarshalNewSessionResult(isres);

		// reset session information
		mSessionId = null;
		mMaxPollResultSize = null;
		mPublisherId = null;
		mSessionId = nsres.getSessionId();
		mPublisherId = nsres.getPublisherId();
		
		if (maxPollResSize != null) {
			mMaxPollResultSize = nsres.getMaxPollResultSize();
			if (mMaxPollResultSize == null)
				throw new CommunicationException("no max-poll-result-size in newSession result");
		}
	}


	@Override
	public void newSession() throws IfmapErrorResult, IfmapException {
		newSession(null);
	}

	@Override
	public void endSession() throws IfmapErrorResult, IfmapException {
		EndSessionRequest esr = mRequestFactory.createEndSessionReq();
		esr.setSessionId(getSessionIdProtected());
		InputStream isreq = mMarshaller.marshalEndSessionRequest(esr);
		InputStream isres = doHttpRequest(isreq);
		mUnmarshaller.checkEndSessionResult(isres);
		mPublisherId = null;
		mSessionId = null;
		mMaxPollResultSize = null;
	}

	@Override
	public void renewSession() throws IfmapErrorResult, IfmapException {
		RenewSessionRequest rsr = mRequestFactory.createRenewSessionReq();
		rsr.setSessionId(getSessionIdProtected());
		InputStream isreq = mMarshaller.marshalRenewSessionRequest(rsr);
		InputStream isres = doHttpRequest(isreq);
		mUnmarshaller.checkRenewSessionResult(isres);
	}
		
	
	@Override
	public void purgePublisher() throws IfmapErrorResult, IfmapException {
		purgePublisher(getPublisherId());
	}

	@Override
	public void purgePublisher(String publisherId) 
			throws IfmapErrorResult, IfmapException {
		PurgePublisherRequest ppr = mRequestFactory.createPurgePublisherReq();
		ppr.setPublisherId(publisherId);
		ppr.setSessionId(getSessionIdProtected());
		InputStream isreq = mMarshaller.marshalPurgePublisherRequest(ppr);
		InputStream isres = doHttpRequest(isreq);
		mUnmarshaller.checkPurgePublisherReceived(isres);
	}

	@Override
	public void publish(PublishRequest pr) throws IfmapErrorResult, IfmapException {
		if (pr == null)
			throw new NullPointerException();
		
		pr.setSessionId(getSessionIdProtected());
		InputStream isreq = mMarshaller.marshalPublishRequest(pr);
		InputStream isres = doHttpRequest(isreq);
		mUnmarshaller.checkPublishReceived(isres);
	}

	@Override
	public void subscribe(SubscribeRequest sr) throws IfmapErrorResult, IfmapException {
		if (sr == null)
			throw new NullPointerException();
		
		sr.setSessionId(getSessionIdProtected());
		InputStream isreq = mMarshaller.marshalSubscribeRequest(sr);
		InputStream isres = doHttpRequest(isreq);
		mUnmarshaller.checkSubscribeReceived(isres);
	}

	@Override
	public SearchResult search(SearchRequest sr) throws IfmapErrorResult, IfmapException {
		if (sr == null)
			throw new NullPointerException();
		
		sr.setSessionId(getSessionIdProtected());
		InputStream isreq = mMarshaller.marshalSearchRequest(sr);
		InputStream isres = doHttpRequest(isreq);
		return mUnmarshaller.unmarshalSearchResult(isres);
	}

	@Override
	public ARC getArc() throws InitializationException {
		ARC ret = null;
		
		if (isBasicAuth())
			ret = new ArcImpl(this, getUrl(), getUser(), getPassword(), mTrustManagers);
		else
			ret = new ArcImpl(this, getUrl(), mKeyManagers, mTrustManagers);
		
		if (usesGzip())
			ret.setGzip(true);
		
		return ret;
	}

	@Override
	public String getSessionId() {
		return mSessionId;
	}

	@Override
	public String getPublisherId() {
		return mPublisherId;
	}

	@Override
	public Integer getMaxPollResSize() {
		return mMaxPollResultSize;
	}

	@Override
	public void setSessionId(String sessionId) {
		mSessionId = sessionId;
	}

	@Override
	public void setPublisherId(String publisherId) {
		mPublisherId = publisherId;
	}

	@Override
	public void setMaxPollResSize(Integer mprs) {
		mMaxPollResultSize = mprs;
	}
	
	private String getSessionIdProtected() throws IfmapException {
		String ret = getSessionId();
		if (ret == null)
			throw new IfmapException("no session-id", "session-id not set for channel");
		return ret;
	}
}
