package de.fhhannover.inform.trust.ifmapj.binding;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.junit.Test;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import de.fhhannover.inform.trust.ifmapj.exception.MarshalException;
import de.fhhannover.inform.trust.ifmapj.exception.UnmarshalException;
import de.fhhannover.inform.trust.ifmapj.messages.EndSessionRequest;
import de.fhhannover.inform.trust.ifmapj.messages.NewSessionRequest;
import de.fhhannover.inform.trust.ifmapj.messages.PollRequest;
import de.fhhannover.inform.trust.ifmapj.messages.PurgePublisherRequest;
import de.fhhannover.inform.trust.ifmapj.messages.RenewSessionRequest;
import de.fhhannover.inform.trust.ifmapj.messages.RequestFactoryImpl;

public class MarshallerTest {

	 private static final String IFMAP_BASE_NS = "http://www.trustedcomputinggroup.org/2010/IFMAP/2";
	 private static final String SOAP_NS = "http://www.w3.org/2003/05/soap-envelope";
	 
	/**
	 * The marshaller to be tested, currently it's the DomMarshaller.
	 */
	private static final Marshaller sMarshaller = BindingFactory.newMarshaller();
	
	/**
	 * Helpers to do some stuff...
	 */
	private RequestFactoryImpl rf;
	private DocumentBuilderFactory dbf;
	DocumentBuilder db;
	{ 
		rf = new RequestFactoryImpl();
		dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		try {
			db = dbf.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			fail();
			e.printStackTrace();
		}
	}
	
	@Test
	public void testMarshalNewSession() throws MarshalException, UnmarshalException {
		Element newSessionEl;
		int childCount;
		// First the simple one
		NewSessionRequest nsr = null;
		Document res = null;
		InputStream isres = null;
		String attrVal = null;
		
		nsr = rf.createNewSessionReq();
		assertNotNull(nsr);
		isres = sMarshaller.marshalNewSessionRequest(nsr);
		assertNotNull(isres);
		res = parseResultStream(isres);
		newSessionEl = checkElementInBody(res, "newSession", IFMAP_BASE_NS);
		childCount = newSessionEl.getChildNodes().getLength();
		
		assertEquals("children in newSession found", 0, childCount);
		assertEquals("attributes found", 0, countNonXmlnsAttributes(newSessionEl));
		
		// Now again with max-poll-result-size
		nsr = rf.createNewSessionReq();
		assertNotNull(nsr);
		nsr.setMaxPollResultSize(1234);
		isres = sMarshaller.marshalNewSessionRequest(nsr);
		assertNotNull(isres);
		res = parseResultStream(isres);
		newSessionEl = checkElementInBody(res, "newSession", IFMAP_BASE_NS);
		childCount = newSessionEl.getChildNodes().getLength();
		assertEquals("children in newSession found", 0, childCount);
		assertEquals("expected single attribute", 1, countNonXmlnsAttributes(newSessionEl));
		attrVal = newSessionEl.getAttribute("max-poll-result-size");
		assertNotNull("no max-poll-result-size attribute found", attrVal);
		assertEquals("unexpected max-poll-result-size value", "1234", attrVal);
	}

	@Test
	public void testMarhsalEndSession() throws MarshalException {
		EndSessionRequest esr = rf.createEndSessionReq();
		esr.setSessionId("1-2-3-4");
		InputStream isres = sMarshaller.marshalEndSessionRequest(esr);
		Document res = parseResultStream(isres);
		Element endSessionEl = checkElementInBody(res, "endSession", IFMAP_BASE_NS);
		checkNoChilds(endSessionEl);
		checkOnlySessionIdAttribute(endSessionEl, "1-2-3-4");

		esr = rf.createEndSessionReq();
		try {
			sMarshaller.marshalEndSessionRequest(esr);
			fail("marshalling with null session-id worked");
		} catch (Exception e) {
			// didn't work, all good
		}
	}

	@Test
	public void testMarshalRenewSession() throws MarshalException {
		RenewSessionRequest rsr = rf.createRenewSessionReq();
		rsr.setSessionId("1234");
		InputStream isres = sMarshaller.marshalRenewSessionRequest(rsr);
		Document res = parseResultStream(isres);
		Element renewSessionEl = checkElementInBody(res, "renewSession", IFMAP_BASE_NS);
		checkNoChilds(renewSessionEl);
		checkOnlySessionIdAttribute(renewSessionEl, "1234");
		
		rsr = rf.createRenewSessionReq();
		try {
			sMarshaller.marshalRenewSessionRequest(rsr);
			fail("marshalling with null session-id worked");
		} catch (Exception e) {
			// didn't work, all good
		}
	}

	@Test
	public void testMarshalPoll() throws MarshalException {
		PollRequest pr = rf.createPollReq();
		pr.setSessionId("1234");
		InputStream isres = sMarshaller.marshalPollRequest(pr);
		Document res = parseResultStream(isres);
		Element renewSessionEl = checkElementInBody(res, "poll", IFMAP_BASE_NS);
		checkNoChilds(renewSessionEl);
		checkOnlySessionIdAttribute(renewSessionEl, "1234");
		
		pr = rf.createPollReq();
		try {
			sMarshaller.marshalPollRequest(pr);
			fail("marshalling with null session-id worked");
		} catch (Exception e) {
			// didn't work, all good
		}
	}

	@Test
	public void testMarshalPurgePublisher() throws MarshalException {
		PurgePublisherRequest ppr = rf.createPurgePublisherReq();;
		ppr.setSessionId("1234");
		ppr.setPublisherId("4321");
		InputStream isres = sMarshaller.marshalPurgePublisherRequest(ppr);
		Document res = parseResultStream(isres);
		Element purgePublisherEl = checkElementInBody(res, "purgePublisher", IFMAP_BASE_NS);
		checkNoChilds(purgePublisherEl);
		checkAtLeastSessionIdAttribute(purgePublisherEl, "1234");
		checkGenericAttribute(purgePublisherEl, "ifmap-publisher-id", "4321");
		
		ppr = rf.createPurgePublisherReq();
		try {
			// no publisher-id and no session-id
			sMarshaller.marshalPurgePublisherRequest(ppr);
			fail("marshalling with null session-id worked");
		} catch (Exception e) {
			// didn't work, all good
		}
		
		ppr.setPublisherId("4321");
		try {
			// only publisher-id
			sMarshaller.marshalPurgePublisherRequest(ppr);
			fail("marshalling with null session-id worked");
		} catch (Exception e) {
			// didn't work, all good
		}
		ppr = rf.createPurgePublisherReq();
		ppr.setSessionId("1234");
		try {
			// only session-id
			sMarshaller.marshalPurgePublisherRequest(ppr);
			fail("marshalling with null session-id worked");
		} catch (Exception e) {
			// didn't work, all good
		}
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                      HELPERS FOR PRESIDENT                              *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	
	private void checkGenericAttribute(Element el, String name, String expectedValue) {
		int attrCount = countNonXmlnsAttributes(el);
		assertTrue("missing attributes", attrCount > 0);
		Attr attrNode = el.getAttributeNode(name);
		assertNotNull("no " + name + " attribute found", attrNode);
		String val = attrNode.getValue();
		assertEquals("wrong value for " + name + " attribute", expectedValue, val);
	}

	private void checkOnlySessionIdAttribute(Element el, String expected) {
		checkAtLeastSessionIdAttribute(el, expected);
		assertEquals("unexpected attribute count", 1, countNonXmlnsAttributes(el));
	}

	private void checkAtLeastSessionIdAttribute(Element el, String expected) {
		checkGenericAttribute(el, "session-id", expected);
	}

	private void checkNoChilds(Element el) {
		int childCount = el.getChildNodes().getLength();
		assertEquals("children in " + el.getLocalName() + " found", 0, childCount);
	}
	
	private int countNonXmlnsAttributes(Element el) {
		int ret = 0;
		NamedNodeMap map = el.getAttributes();
		for (int i = 0; i < map.getLength(); i++) {
			Attr attrNode = null;
			if (map.item(i).getNodeType() == Node.ATTRIBUTE_NODE) {
				attrNode = (Attr)map.item(i);
				if (attrNode.getPrefix() != null		// skip namespace decls
						&& attrNode.getPrefix().equals("xmlns")) {
					continue;
				}
				ret++;
			}
		}
		return ret;
	}
	
	private Element checkElementInBody(Document res, String elName, String uri) {
		
		Element root = null;
		Element body = null;
		// only a single root element
		assertEquals("no single root element", 1, res.getChildNodes().getLength());
		root = findSingleElementInList(res.getChildNodes(), "Envelope", SOAP_NS);
		assertTrue("Envelope has no children", root.getChildNodes().getLength() > 0);
		body = findSingleElementInList(root.getChildNodes(), "Body", SOAP_NS);
		return findSingleElementInList(body.getChildNodes(), elName, IFMAP_BASE_NS);
	}
	
	private Element findSingleElementInList(NodeList list, String name, String uri) {
		Element ret = null;
		for (int i = 0; i < list.getLength(); i++) {
			Element child = null;
			// only elements
			if (list.item(i).getNodeType() != Node.ELEMENT_NODE)
				continue;
			
			child = (Element)list.item(i);
			
			if (child.getNamespaceURI().equals(uri)
					&& child.getLocalName().equals(name)) {
				assertNull("multiple " + name + " elements found", ret);
				ret = child;
			}
		}
		assertNotNull("no " + name + " element found", ret);
		return ret;
	}

	private Document parseResultStream(InputStream is) {
		try {
			return db.parse(is);
		} catch (SAXException e) {
			e.printStackTrace();
			fail(e.getMessage());
		} catch (IOException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		fail();
		return null;
	}
}
